from aurelio import utils


class Dict:
    def __init__(self, settings):
        self.__client = utils.client_factory(settings)

    @property
    def client(self):
        return self.__client.client

    def __setitem__(self, key, value):
        self.__client[key] = value

    def __getitem__(self, key):
        return self.__client[key]

    def __len__(self):
        return len(self.__client)

    def __delitem__(self, key):
        del self.__client[key]

    def clear(self):
        self.__client.clear()

    def __contains__(self, key):
        return key in self.__client

    def has_key(self, key):
        return key in self.__client

    def keys(self):
        return self.__client.keys()

    def items(self):
        return self.__client.items()

    def values(self):
        for _, value in self.items():
            yield value

    def pop(self, key):
        return self.__client.pop(key)

    def __iter__(self):
        return iter(self.values())

    def copy(self):
        raise NotImplementedError('Copy method is not implemented!')

    def __cmp__(self, other):
        raise NotImplementedError('Compare method is not implemented!')

    def update(self, *args, **kwargs):
        raise NotImplementedError('Update method is not implemented!')



from abc import ABC, abstractmethod


class AbstractClient(ABC):
    def __init__(self):
        self.__client = None

    @property
    def client(self):
        return self.__client

    @abstractmethod
    def __setitem__(self, key, value):
        pass

    @abstractmethod
    def __getitem__(self, key):
        pass

    @abstractmethod
    def __len__(self):
        pass

    @abstractmethod
    def __delitem__(self, key):
        pass

    @abstractmethod
    def clear(self):
        pass

    @abstractmethod
    def __contains__(self, key):
        pass

    @abstractmethod
    def keys(self):
        pass

    @abstractmethod
    def items(self):
        pass

    @abstractmethod
    def pop(self, key):
        pass



from aurelio.clients import RedisClient
from aurelio.exceptions import ClientError
from aurelio.settings import RedisSettings


def client_factory(settings):
    mapping = {
        'RedisSettings': RedisClient,
    }
    try:
        return mapping[settings.__class__.__name__](settings)
    except KeyError:
        raise ClientError('Client not supported!')



import dataclasses


@dataclasses.dataclass
class RedisSettings:
    host: str = 'localhost'
    port: int = 6379
    db: int = 0
    password: str = None
    socket_timeout = None
    socket_connect_timeout = None
    socket_keepalive = None
    socket_keepalive_options = None
    connection_pool = None
    unix_socket_path = None
    encoding: str = 'utf-8'
    encoding_errors: str = 'strict'
    charset = None
    errors = None
    decode_responses: bool = False
    retry_on_timeout: bool = False
    retry_on_error = []
    ssl: bool = False
    ssl_keyfile = None
    ssl_certfile = None
    ssl_cert_reqs: str = 'required'
    ssl_ca_certs = None
    ssl_ca_path = None
    ssl_check_hostname: bool = False
    ssl_password = None
    ssl_validate_ocsp: bool = False
    ssl_validate_ocsp_stapled: bool = False
    ssl_ocsp_context = None
    ssl_ocsp_expected_cert = None
    max_connections = None
    single_connection_client: bool = False
    health_check_interval: int = 0
    client_name = None
    username: str = None
    retry = None
    redis_connect_func = None

    def asdict(self):
        return dataclasses.asdict(self)



import logging

from aurelio.clients.abstract_client import AbstractClient

try:
    from redis import Redis
except ImportError:
    logging.warning('Could not import Redis client!')


class RedisClient(AbstractClient):
    def __init__(self, settings):
        super().__init__()
        self.__settings = settings
        self.__client = Redis(**settings.asdict())

    def decode(self, data):
        return data.decode(self.__settings.encoding)

    def __setitem__(self, key, value):
        self.__client.set(key, value)

    def __getitem__(self, key):
        value = self.__client.get(key)
        if value is None:
            raise KeyError(key)
        return self.decode(value)

    def __len__(self):
        return self.__client.dbsize()

    def __delitem__(self, key):
        self.pop(key)

    def clear(self):
        self.__client.flushdb()

    def __contains__(self, key):
        return self.__client.exists(key)

    def keys(self):
        for key in self.__client.keys():
            yield self.decode(key)

    def items(self):
        for key in self.__client.scan_iter():
            yield (self.decode(key), self[key])

    def pop(self, key):
        value = self[key]
        self.__client.delete(key)
        return value
